require 'httpclient'
require 'json'
require_relative 'response'
require 'base64'

module MxHero
  module API
    module Communication

      # Make a HTTP call
      # @param method [Symbol] indicate the HTTP verb (:get, :post, :put, etc)
      # @param url [String]
      # @param body [String] (default: nil)
      # @param [Hash] more_options
      # @option more_options [Boolean] :throw_exception (default: true) throw exception if the response status are between 500 and 600
      # @return [HTTP::Message](http://www.rubydoc.info/gems/httpclient/HTTP/Message)
      def call(method, url, body = nil, more_options = {})
        unless @client
          @client ||= HTTPClient.new(:force_basic_auth => true)
        end
        @client.set_auth(url, @username, @password)
        response = @client.request(method, url, nil, body, headers(more_options))
        raise "Unauthorized" if response.status == 401
        unless more_options[:throw_exception] == false
          raise "An error ocurred when try to communicate with the API\nError: #{response.inspect}" if (500..600).include?(response.status)
        end
        response
      end

      # Default headers
			def headers(options = {})
				accept = options.fetch(:accept, 'application/json')
				content_type = options.fetch(:content_type, 'application/json')
				return {
          'Accept' => accept,
          'Content-Type' => content_type,
          "Authorization" => "Basic " + ::Base64.encode64(@username + ':' + @password).gsub("\n",''),
        }.merge(@as_user ? { "X-MxHero-As-User" => @as_user } : {})
      end

      # @return [Hash]
      def json_parse(json)
        JSON.parse(json, symbolize_names: true)
      end

    end
  end
end
