# DTO class allows to create a structure that support
# export to json and construct with a hash.
#
# Example of use:
#
#     class User < DTO.new(:first_name, :last_name, :birthday)
#
#     end
#
# or
#
#     User = Class.new(DTO.new :first_name, :last_name, :birthday)
#
class DTO < Struct

  def initialize(attrs = {})
    members.each do |member|
      send("#{member}=", attrs[member])
    end
  end
  
  def to_json
    attrs = {}
    members.each { |attr| attrs[attr] = send(attr) }
    attrs.to_json
  end

  def to_s
    members.map { |attr| "#{attr}: #{send(attr)}" }.join(", ")
  end

end
