#require 'struct'

module MxHero
  module API

    # The class that contains the response information
    #
    # The *code* represent the HTTP code of the response.
    #
    # The *msg* represent a hash with the response information. Example:
    #
    #     { status: 500, code: 500,
    #       developerMessage: "rules.already.exists.for.component",
    #       moreInfoUrl: "mailto:support@mxhero.com" }
    #
    class Response < Struct.new(:code, :msg)

      # Response is successful? Based in HTTP status code
      def success?
        code.to_i == 200 || code.to_i == 201
      end

      # @return [String] the error message
      def error
        content[:developerMessage] if content.is_a? Hash
      end

      alias_method :content, :msg
      alias_method :content=, :msg=
    end

  end
end
